/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.elements;

import com.liferay.poshi.runner.util.Dom4JUtil;
import com.liferay.poshi.runner.util.FileUtil;

import java.io.File;
import java.io.IOException;

import java.net.URISyntaxException;
import java.net.URL;

import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.Element;

/**
 * @author Kenji Heigel
 */
public class PoshiElementFactory {

	public static PoshiElement newPoshiElement(Element element) {
		for (PoshiElement poshiElement : _poshiElements) {
			PoshiElement newPoshiElement = poshiElement.clone(element);

			if (newPoshiElement != null) {
				return newPoshiElement;
			}
		}

		String xml = null;

		try {
			xml = Dom4JUtil.format(element);
		}
		catch (IOException ioe) {
			xml = element.toString();
		}

		throw new RuntimeException("Unknown element\n" + xml);
	}

	public static PoshiElement newPoshiElement(
		PoshiElement parentPoshiElement, String readableSyntax) {

		for (PoshiElement poshiElement : _poshiElements) {
			PoshiElement newPoshiElement = poshiElement.clone(
				parentPoshiElement, readableSyntax);

			if (newPoshiElement != null) {
				return newPoshiElement;
			}
		}

		throw new RuntimeException(
			"Unknown readable syntax\n" + readableSyntax);
	}

	public static PoshiElement newPoshiElementFromFile(String filePath) {
		File file = new File(filePath);

		try {
			String content = FileUtil.read(file);

			if (content.contains("<definition")) {
				Document document = Dom4JUtil.parse(content);

				Element rootElement = document.getRootElement();

				return newPoshiElement(rootElement);
			}

			return newPoshiElement(null, content);
		}
		catch (Exception e) {
			System.out.println("Unable to generate the Poshi element");

			e.printStackTrace();
		}

		return null;
	}

	private static final List<PoshiElement> _poshiElements = new ArrayList<>();

	static {
		try {
			URL url = BasePoshiElement.class.getResource(
				BasePoshiElement.class.getSimpleName() + ".class");

			File basePoshiElementClassFile = new File(url.toURI());

			File dir = basePoshiElementClassFile.getParentFile();

			for (File file : dir.listFiles()) {
				String fileName = file.getName();

				if (fileName.endsWith(
						BasePoshiElement.class.getSimpleName() + ".class")) {

					continue;
				}

				if (!fileName.endsWith("PoshiElement.class")) {
					continue;
				}

				Package pkg = BasePoshiElement.class.getPackage();

				int index = fileName.indexOf(".");

				String className = fileName.substring(0, index);

				Class<?> clazz = Class.forName(pkg.getName() + "." + className);

				if (BasePoshiElement.class.isAssignableFrom(clazz)) {
					PoshiElement poshiElement =
						(PoshiElement)clazz.newInstance();

					_poshiElements.add(poshiElement);
				}
			}
		}
		catch (ClassNotFoundException | IllegalAccessException |
			   InstantiationException | URISyntaxException e) {

			throw new RuntimeException(e);
		}
	}

}