/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.poshi.runner.util;

import com.jayway.jsonpath.DocumentContext;
import com.jayway.jsonpath.JsonPath;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * @author Michael Hashimoto
 */
public class JSONCurlUtil {

	public static String get(String curlOptions, String jsonPath)
		throws Exception {

		Runtime runtime = Runtime.getRuntime();

		StringBuilder sb = new StringBuilder();

		curlOptions = curlOptions.trim();

		curlOptions = curlOptions.replaceAll("\\s+\\\\?\\s+", " ");

		for (String curlOption : curlOptions.split(" ")) {
			if (curlOption.matches("https?:\\/\\/.+")) {
				sb.append(curlOption);
			}
			else if (curlOption.matches("[^\\=]*\\=[^\\=]*")) {
				int x = curlOption.indexOf("=");

				String name = curlOption.substring(0, x);
				String value = curlOption.substring(x + 1);

				if (value.startsWith("{") && value.endsWith("}") &&
					OSDetector.isWindows()) {

					value = value.replaceAll("\"", "\\\\\"");

					value = "\"" + value + "\"";
				}

				sb.append(" ");
				sb.append(name);
				sb.append("=");
				sb.append(value);
			}
			else {
				sb.append(" ");
				sb.append(curlOption);
			}
		}

		System.out.println("curl " + sb.toString());

		Process process = runtime.exec("curl " + sb.toString());

		InputStreamReader inputStreamReader = new InputStreamReader(
			process.getInputStream());

		BufferedReader inputBufferedReader = new BufferedReader(
			inputStreamReader);

		String line = null;

		sb = new StringBuilder();

		while ((line = inputBufferedReader.readLine()) != null) {
			sb.append(line);
		}

		System.out.println(sb.toString());

		try {
			DocumentContext documentContext = JsonPath.parse(sb.toString());

			Object object = documentContext.read(jsonPath);

			return object.toString();
		}
		catch (Exception e) {
			return "";
		}
	}

}